@@ -4,7 +4,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: main.c,v 1.18 2004/12/13 13:58:56 holtmann Exp $
+ *  $Id: main.c,v 1.21 2005/01/28 17:06:35 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -64,9 +64,6 @@ static struct device_list *device_list = NULL;
 
 static GMainLoop *event_loop;
 
-gboolean io_stack_event(GIOChannel *chan, GIOCondition cond, gpointer data);
-gboolean io_security_event(GIOChannel *chan, GIOCondition cond, gpointer data);
-
 static void usage(void)
 {
 	printf("hcid - HCI daemon ver %s\n", VERSION);
@@ -87,8 +84,8 @@ struct device_opts *alloc_device_opts(char *ref)
 
 	device = malloc(sizeof(struct device_list));
 	if (!device) {
-		syslog(LOG_INFO, "Can't allocate devlist opts buffer. %s(%d)",
-			strerror(errno), errno);
+		syslog(LOG_INFO, "Can't allocate devlist opts buffer: %s (%d)",
+							strerror(errno), errno);
 		exit(1);
 	}
 
@@ -164,6 +161,7 @@ static void configure_device(int hdev)
 {
 	struct device_opts *device_opts;
 	struct hci_dev_req dr;
+	struct hci_dev_info di;
 	int s;
 
 	/* Do configuration in the separate process */
@@ -171,8 +169,8 @@ static void configure_device(int hdev)
 		case 0:
 			break;
 		case -1:
-			syslog(LOG_ERR, "Fork failed. Can't init device hci%d. %s(%d)\n",
-					hdev, strerror(errno), errno);
+			syslog(LOG_ERR, "Fork failed. Can't init device hci%d: %s (%d)",
+						hdev, strerror(errno), errno);
 		default:
 			return;
 	}
@@ -180,18 +178,26 @@ static void configure_device(int hdev)
 	set_title("hci%d config", hdev);
 
 	if ((s = hci_open_dev(hdev)) < 0) {
-		syslog(LOG_ERR, "Can't open device hci%d. %s(%d)\n", hdev, strerror(errno), errno);
+		syslog(LOG_ERR, "Can't open device hci%d: %s (%d)",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 
+	di.dev_id = hdev;
+	if (ioctl(s, HCIGETDEVINFO, (void *) &di) < 0)
+		exit(1);
+
+	if (hci_test_bit(HCI_RAW, &di.flags))
+		exit(0);
+
 	dr.dev_id   = hdev;
 	device_opts = get_device_opts(s, hdev);
 
 	/* Set scan mode */
 	dr.dev_opt = device_opts->scan;
 	if (ioctl(s, HCISETSCAN, (unsigned long) &dr) < 0) {
-		syslog(LOG_ERR, "Can't set scan mode on hci%d. %s(%d)\n",
-				hdev, strerror(errno), errno);
+		syslog(LOG_ERR, "Can't set scan mode on hci%d: %s (%d)",
+						hdev, strerror(errno), errno);
 	}
 
 	/* Set authentication */
@@ -201,8 +207,8 @@ static void configure_device(int hdev)
 		dr.dev_opt = AUTH_DISABLED;
 
 	if (ioctl(s, HCISETAUTH, (unsigned long) &dr) < 0) {
-		syslog(LOG_ERR, "Can't set auth on hci%d. %s(%d)\n",
-				hdev, strerror(errno), errno);
+		syslog(LOG_ERR, "Can't set auth on hci%d: %s (%d)",
+						hdev, strerror(errno), errno);
 	}
 
 	/* Set encryption */
@@ -212,8 +218,8 @@ static void configure_device(int hdev)
 		dr.dev_opt = ENCRYPT_DISABLED;
 
 	if (ioctl(s, HCISETENCRYPT, (unsigned long) &dr) < 0) {
-		syslog(LOG_ERR, "Can't set encrypt on hci%d. %s(%d)\n",
-				hdev, strerror(errno), errno);
+		syslog(LOG_ERR, "Can't set encrypt on hci%d: %s (%d)",
+						hdev, strerror(errno), errno);
 	}
 
 	/* Set device class */
@@ -243,6 +249,7 @@ static void init_device(int hdev)
 {
 	struct device_opts *device_opts;
 	struct hci_dev_req dr;
+	struct hci_dev_info di;
 	int s;
 
 	/* Do initialization in the separate process */
@@ -250,8 +257,8 @@ static void init_device(int hdev)
 		case 0:
 			break;
 		case -1:
-			syslog(LOG_ERR, "Fork failed. Can't init device hci%d. %s(%d)\n",
-					hdev, strerror(errno), errno);
+			syslog(LOG_ERR, "Fork failed. Can't init device hci%d: %s (%d)",
+						hdev, strerror(errno), errno);
 		default:
 			return;
 	}
@@ -259,18 +266,25 @@ static void init_device(int hdev)
 	set_title("hci%d init", hdev);
 
 	if ((s = hci_open_dev(hdev)) < 0) {
-		syslog(LOG_ERR, "Can't open device hci%d. %s(%d)\n",
-					hdev, strerror(errno), errno);
+		syslog(LOG_ERR, "Can't open device hci%d: %s (%d)",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 
 	/* Start HCI device */
 	if (ioctl(s, HCIDEVUP, hdev) < 0 && errno != EALREADY) {
-		syslog(LOG_ERR, "Can't init device hci%d. %s(%d)\n", hdev, 
-				strerror(errno), errno);
+		syslog(LOG_ERR, "Can't init device hci%d: %s (%d)",
+						hdev, strerror(errno), errno);
 		exit(1);
 	}
 
+	di.dev_id = hdev;
+	if (ioctl(s, HCIGETDEVINFO, (void *) &di) < 0)
+		exit(1);
+
+	if (hci_test_bit(HCI_RAW, &di.flags))
+		exit(0);
+
 	dr.dev_id   = hdev;
 	device_opts = get_device_opts(s, hdev);
 
@@ -278,8 +292,8 @@ static void init_device(int hdev)
 	if (device_opts->pkt_type) {
 		dr.dev_opt = device_opts->pkt_type;
 		if (ioctl(s, HCISETPTYPE, (unsigned long) &dr) < 0) {
-			syslog(LOG_ERR, "Can't set packet type on hci%d. %s(%d)\n",
-				hdev, strerror(errno), errno);
+			syslog(LOG_ERR, "Can't set packet type on hci%d: %s (%d)",
+						hdev, strerror(errno), errno);
 		}
 	}
 
@@ -287,8 +301,8 @@ static void init_device(int hdev)
 	if (device_opts->link_mode) {
 		dr.dev_opt = device_opts->link_mode;
 		if (ioctl(s, HCISETLINKMODE, (unsigned long) &dr) < 0) {
-			syslog(LOG_ERR, "Can't set link mode on hci%d. %s(%d)\n",
-				hdev, strerror(errno), errno);
+			syslog(LOG_ERR, "Can't set link mode on hci%d: %s (%d)",
+						hdev, strerror(errno), errno);
 		}
 	}
 
@@ -296,8 +310,8 @@ static void init_device(int hdev)
 	if (device_opts->link_policy) {
 		dr.dev_opt = device_opts->link_policy;
 		if (ioctl(s, HCISETLINKPOL, (unsigned long) &dr) < 0) {
-			syslog(LOG_ERR, "Can't set link policy on hci%d. %s(%d)\n",
-				hdev, strerror(errno), errno);
+			syslog(LOG_ERR, "Can't set link policy on hci%d: %s (%d)",
+						hdev, strerror(errno), errno);
 		}
 	}
 
@@ -311,16 +325,16 @@ static void init_all_devices(int ctl)
 	int i;
 
 	if (!(dl = malloc(HCI_MAX_DEV * sizeof(struct hci_dev_req) + sizeof(uint16_t)))) {
-		syslog(LOG_INFO, "Can't allocate devlist buffer. %s(%d)", 
-			strerror(errno), errno);
+		syslog(LOG_INFO, "Can't allocate devlist buffer: %s (%d)",
+							strerror(errno), errno);
 		exit(1);
 	}
 	dl->dev_num = HCI_MAX_DEV;
 	dr = dl->dev_req;
 
 	if (ioctl(ctl, HCIGETDEVLIST, (void *) dl) < 0) {
-		syslog(LOG_INFO, "Can't get device list. %s(%d)",
-			strerror(errno), errno);
+		syslog(LOG_INFO, "Can't get device list: %s (%d)",
+							strerror(errno), errno);
 		exit(1);
 	}
 
@@ -406,7 +420,7 @@ static inline void device_event(GIOChannel *chan, evt_stack_internal *si)
 	}
 }
 
-gboolean io_stack_event(GIOChannel *chan, GIOCondition cond, gpointer data)
+static gboolean io_stack_event(GIOChannel *chan, GIOCondition cond, gpointer data)
 {
 	unsigned char buf[HCI_MAX_FRAME_SIZE], *ptr;
 	evt_stack_internal *si;
@@ -421,8 +435,8 @@ gboolean io_stack_event(GIOChannel *chan, GIOCondition cond, gpointer data)
 		if (err == G_IO_ERROR_AGAIN)
 			return TRUE;
 
-		syslog(LOG_ERR, "Read from control socket failed. %s(%d)",
-				strerror(errno), errno);
+		syslog(LOG_ERR, "Read from control socket failed: %s (%d)",
+							strerror(errno), errno);
 		g_main_quit(event_loop);
 		return FALSE;
 	}
@@ -530,7 +544,8 @@ int main(int argc, char *argv[], char *env[])
 
 	/* Create and bind HCI socket */
 	if ((hcid.sock = socket(AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI)) < 0) {
-		syslog(LOG_ERR, "Can't open HCI socket. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Can't open HCI socket: %s (%d)",
+							strerror(errno), errno);
 		exit(1);
 	}
 
@@ -539,14 +554,16 @@ int main(int argc, char *argv[], char *env[])
 	hci_filter_set_ptype(HCI_EVENT_PKT, &flt);
 	hci_filter_set_event(EVT_STACK_INTERNAL, &flt);
 	if (setsockopt(hcid.sock, SOL_HCI, HCI_FILTER, &flt, sizeof(flt)) < 0) {
-		syslog(LOG_ERR, "Can't set filter. %s(%d)", strerror(errno), errno);
+		syslog(LOG_ERR, "Can't set filter: %s (%d)",
+							strerror(errno), errno);
 		exit(1);
 	}
 
 	addr.hci_family = AF_BLUETOOTH;
 	addr.hci_dev = HCI_DEV_NONE;
 	if (bind(hcid.sock, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
-		syslog(LOG_ERR, "Can't bind HCI socket. %s(%d)\n", strerror(errno), errno);
+		syslog(LOG_ERR, "Can't bind HCI socket: %s (%d)",
+							strerror(errno), errno);
 		exit(1);
 	}
 
