@@ -118,7 +118,7 @@
  *
  *  Copyright (C) 2000-2001  Qualcomm Incorporated
  *  Copyright (C) 2002-2003  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2002-2004  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2002-2005  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -139,7 +139,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: parser.y,v 1.10 2004/12/13 16:59:04 holtmann Exp $
+ *  $Id: parser.y,v 1.12 2005/01/27 17:47:34 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1677,7 +1677,7 @@ yyreturn:
 
 int yyerror(char *s) 
 {
-	syslog(LOG_ERR, "%s line %d\n", s, lineno);
+	syslog(LOG_ERR, "%s line %d", s, lineno);
 	return 0;
 }
 
