@@ -23,7 +23,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: main.c,v 1.10 2004/08/10 12:59:39 holtmann Exp $
+ *  $Id: main.c,v 1.15 2004/12/25 17:43:17 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -72,21 +72,23 @@ static int l2cap_connect(bdaddr_t *src, bdaddr_t *dst, unsigned short psm)
 	if ((sk = socket(PF_BLUETOOTH, SOCK_SEQPACKET, BTPROTO_L2CAP)) < 0)
 		return -1;
 
+	memset(&addr, 0, sizeof(addr));
 	addr.l2_family  = AF_BLUETOOTH;
 	bacpy(&addr.l2_bdaddr, src);
-	addr.l2_psm = 0;
 
 	if (bind(sk, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
 		close(sk);
 		return -1;
 	}
 
+	memset(&opts, 0, sizeof(opts));
 	opts.imtu = HIDP_DEFAULT_MTU;
 	opts.omtu = HIDP_DEFAULT_MTU;
 	opts.flush_to = 0xffff;
 
 	setsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &opts, sizeof(opts));
 
+	memset(&addr, 0, sizeof(addr));
 	addr.l2_family  = AF_BLUETOOTH;
 	bacpy(&addr.l2_bdaddr, dst);
 	addr.l2_psm = htobs(psm);
@@ -99,15 +101,16 @@ static int l2cap_connect(bdaddr_t *src, bdaddr_t *dst, unsigned short psm)
 	return sk;
 }
 
-static int l2cap_listen(const bdaddr_t *bdaddr, unsigned short psm, int backlog)
+static int l2cap_listen(const bdaddr_t *bdaddr, unsigned short psm, int lm, int backlog)
 {
 	struct sockaddr_l2 addr;
 	struct l2cap_options opts;
-	int sk, lm = L2CAP_LM_MASTER;
+	int sk;
 
 	if ((sk = socket(PF_BLUETOOTH, SOCK_SEQPACKET, BTPROTO_L2CAP)) < 0)
 		return -1;
 
+	memset(&addr, 0, sizeof(addr));
 	addr.l2_family = AF_BLUETOOTH;
 	bacpy(&addr.l2_bdaddr, bdaddr);
 	addr.l2_psm = htobs(psm);
@@ -119,6 +122,7 @@ static int l2cap_listen(const bdaddr_t *bdaddr, unsigned short psm, int backlog)
 
 	setsockopt(sk, SOL_L2CAP, L2CAP_LM, &lm, sizeof(lm));
 
+	memset(&opts, 0, sizeof(opts));
 	opts.imtu = HIDP_DEFAULT_MTU;
 	opts.omtu = HIDP_DEFAULT_MTU;
 	opts.flush_to = 0xffff;
@@ -151,7 +155,79 @@ static int l2cap_accept(int sk, bdaddr_t *bdaddr)
 	return nsk;
 }
 
-static int create_device(int ctl, int csk, int isk, uint8_t subclass, int timeout)
+static int request_authentication(bdaddr_t *src, bdaddr_t *dst)
+{
+	struct hci_conn_info_req *cr;
+	char addr[18];
+	int err, dd, dev_id;
+
+	ba2str(src, addr);
+	dev_id = hci_devid(addr);
+	if (dev_id < 0)
+		return dev_id;
+
+	dd = hci_open_dev(dev_id);
+	if (dd < 0)
+		return dd;
+
+	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
+	if (!cr)
+		return -ENOMEM;
+
+	bacpy(&cr->bdaddr, dst);
+	cr->type = ACL_LINK;
+	err = ioctl(dd, HCIGETCONNINFO, (unsigned long) cr);
+	if (err < 0) {
+		free(cr);
+		hci_close_dev(dd);
+		return err;
+	}
+
+	err = hci_authenticate_link(dd, htobs(cr->conn_info->handle), 25000);
+
+	free(cr);
+	hci_close_dev(dd);
+
+	return err;
+}
+
+static int request_encryption(bdaddr_t *src, bdaddr_t *dst)
+{
+	struct hci_conn_info_req *cr;
+	char addr[18];
+	int err, dd, dev_id;
+
+	ba2str(src, addr);
+	dev_id = hci_devid(addr);
+	if (dev_id < 0)
+		return dev_id;
+
+	dd = hci_open_dev(dev_id);
+	if (dd < 0)
+		return dd;
+
+	cr = malloc(sizeof(*cr) + sizeof(struct hci_conn_info));
+	if (!cr)
+		return -ENOMEM;
+
+	bacpy(&cr->bdaddr, dst);
+	cr->type = ACL_LINK;
+	err = ioctl(dd, HCIGETCONNINFO, (unsigned long) cr);
+	if (err < 0) {
+		free(cr);
+		hci_close_dev(dd);
+		return err;
+	}
+
+	err = hci_encrypt_link(dd, htobs(cr->conn_info->handle), 1, 25000);
+
+	free(cr);
+	hci_close_dev(dd);
+
+	return err;
+}
+
+static int create_device(int ctl, int csk, int isk, uint8_t subclass, int nosdp, int encrypt, int timeout)
 {
 	struct hidp_connadd_req req;
 	struct sockaddr_l2 addr;
@@ -182,9 +258,27 @@ static int create_device(int ctl, int csk, int isk, uint8_t subclass, int timeou
 	req.flags     = 0;
 	req.idle_to   = timeout * 60;
 
-	err = get_hid_device_info(&src, &dst, &req);
-	if (err < 0)
-		goto error;
+	if (!nosdp) {
+		err = get_hid_device_info(&src, &dst, &req);
+		if (err < 0)
+			goto error;
+	} else {
+		struct l2cap_conninfo conn;
+		socklen_t size;
+		uint8_t class[3];
+
+		memset(&conn, 0, sizeof(conn));
+		size = sizeof(conn);
+		if (getsockopt(csk, SOL_L2CAP, L2CAP_CONNINFO, &conn, &size) < 0)
+			memset(class, 0, 3);
+		else
+			memcpy(class, conn.dev_class, 3);
+
+		if (class[1] == 0x25 && (class[2] == 0x00 || class[2] == 0x01))
+			req.subclass = class[0];
+		else
+			req.subclass = 0xc0;
+	}
 
 	if (subclass != 0x00)
 		req.subclass = subclass;
@@ -192,7 +286,16 @@ static int create_device(int ctl, int csk, int isk, uint8_t subclass, int timeou
 	ba2str(&dst, bda);
 	syslog(LOG_INFO, "New HID device %s (%s)", bda, req.name);
 
-	if (req.subclass & 0x40) {
+	if (encrypt && (req.subclass & 0x40)) {
+		err = request_authentication(&src, &dst);
+		if (err < 0) {
+			syslog(LOG_ERR, "Authentication for %s failed", bda);
+			goto error;
+		}
+
+		err = request_encryption(&src, &dst);
+		if (err < 0)
+			syslog(LOG_ERR, "Encryption for %s failed", bda);
 	}
 
 	err = ioctl(ctl, HIDPCONNADD, &req);
@@ -204,7 +307,7 @@ error:
 	return err;
 }
 
-static void run_server(int ctl, int csk, int isk, uint8_t subclass, int timeout)
+static void run_server(int ctl, int csk, int isk, uint8_t subclass, int nosdp, int encrypt, int timeout)
 {
 	struct pollfd p[2];
 	short events;
@@ -230,7 +333,7 @@ static void run_server(int ctl, int csk, int isk, uint8_t subclass, int timeout)
 			ncsk = l2cap_accept(csk, NULL);
 			nisk = l2cap_accept(isk, NULL);
 
-			err = create_device(ctl, ncsk, nisk, subclass, timeout);
+			err = create_device(ctl, ncsk, nisk, subclass, nosdp, encrypt, timeout);
 			if (err < 0)
 				syslog(LOG_ERR, "HID create error %d (%s)",
 						errno, strerror(errno));
@@ -294,7 +397,7 @@ static void do_show(int ctl)
 	}
 }
 
-static void do_connect(int ctl, bdaddr_t *src, bdaddr_t *dst, uint8_t subclass, int timeout)
+static void do_connect(int ctl, bdaddr_t *src, bdaddr_t *dst, uint8_t subclass, int nosdp, int encrypt, int timeout)
 {
 	int csk, isk, err;
 
@@ -313,7 +416,7 @@ static void do_connect(int ctl, bdaddr_t *src, bdaddr_t *dst, uint8_t subclass,
 		exit(1);
 	}
 
-	err = create_device(ctl, csk, isk, subclass, timeout);
+	err = create_device(ctl, csk, isk, subclass, nosdp, encrypt, timeout);
 	if (err < 0) {
 		fprintf(stderr, "HID create error %d (%s)\n",
 						errno, strerror(errno));
@@ -325,7 +428,7 @@ static void do_connect(int ctl, bdaddr_t *src, bdaddr_t *dst, uint8_t subclass,
 	}
 }
 
-static void do_search(int ctl, bdaddr_t *bdaddr, uint8_t subclass, int timeout)
+static void do_search(int ctl, bdaddr_t *bdaddr, uint8_t subclass, int nosdp, int encrypt, int timeout)
 {
 	inquiry_info *info = NULL;
 	bdaddr_t src, dst;
@@ -356,7 +459,7 @@ static void do_search(int ctl, bdaddr_t *bdaddr, uint8_t subclass, int timeout)
 			ba2str(&dst, addr);
 
 			printf("\tConnecting to device %s\n", addr);
-			do_connect(ctl, &src, &dst, subclass, timeout);
+			do_connect(ctl, &src, &dst, subclass, nosdp, encrypt, timeout);
 		}
 	}
 
@@ -411,7 +514,7 @@ static void do_kill(int ctl, bdaddr_t *bdaddr, uint32_t flags)
 
 static void usage(void)
 {
-	printf("hidd - Bluetooth HID daemon\n\n");
+	printf("hidd - Bluetooth HID daemon version %s\n\n", VERSION);
 
 	printf("Usage:\n"
 		"\thidd [options] [commands]\n"
@@ -442,6 +545,9 @@ static struct option main_options[] = {
 	{ "subclass",	1, 0, 'b' },
 	{ "timeout",	1, 0, 't' },
 	{ "device",	1, 0, 'i' },
+	{ "master",	0, 0, 'M' },
+	{ "encrypt",	0, 0, 'E' },
+	{ "nosdp",	0, 0, 'D' },
 	{ "show",	0, 0, 'l' },
 	{ "list",	0, 0, 'l' },
 	{ "server",	0, 0, 'd' },
@@ -467,11 +573,11 @@ int main(int argc, char *argv[])
 	char addr[18];
 	int log_option = LOG_NDELAY | LOG_PID;
 	int opt, fd, ctl, csk, isk;
-	int mode = 0, daemon = 1, timeout = 30;
+	int mode = 0, daemon = 1, nosdp = 0, encrypt = 0, timeout = 30, lm = 0;
 
 	bacpy(&bdaddr, BDADDR_ANY);
 
-	while ((opt = getopt_long(argc, argv, "+i:nt:b:ldsc:k:Ku:h", main_options, NULL)) != -1) {
+	while ((opt = getopt_long(argc, argv, "+i:nt:b:MEDldsc:k:Ku:h", main_options, NULL)) != -1) {
 		switch(opt) {
 		case 'i':
 			if (!strncasecmp(optarg, "hci", 3))
@@ -491,6 +597,15 @@ int main(int argc, char *argv[])
 			else
 				subclass = atoi(optarg);
 			break;
+		case 'M':
+			lm |= L2CAP_LM_MASTER;
+			break;
+		case 'E':
+			encrypt = 1;
+			break;
+		case 'D':
+			nosdp = 1;
+			break;
 		case 'l':
 			mode = 0;
 			break;
@@ -535,14 +650,14 @@ int main(int argc, char *argv[])
 
 	switch (mode) {
 	case 1:
-		csk = l2cap_listen(&bdaddr, L2CAP_PSM_HIDP_CTRL, 10);
+		csk = l2cap_listen(&bdaddr, L2CAP_PSM_HIDP_CTRL, lm, 10);
 		if (csk < 0) {
 			perror("Can't listen on HID control channel");
 			close(ctl);
 			exit(1);
 		}
 
-		isk = l2cap_listen(&bdaddr, L2CAP_PSM_HIDP_INTR, 10);
+		isk = l2cap_listen(&bdaddr, L2CAP_PSM_HIDP_INTR, lm, 10);
 		if (isk < 0) {
 			perror("Can't listen on HID interrupt channel");
 			close(ctl);
@@ -552,12 +667,12 @@ int main(int argc, char *argv[])
 		break;
 
 	case 2:
-		do_search(ctl, &bdaddr, subclass, timeout);
+		do_search(ctl, &bdaddr, subclass, nosdp, encrypt, timeout);
 		close(ctl);
 		exit(0);
 
 	case 3:
-		do_connect(ctl, &bdaddr, &dev, subclass, timeout);
+		do_connect(ctl, &bdaddr, &dev, subclass, nosdp, encrypt, timeout);
 		close(ctl);
 		exit(0);
 
@@ -605,7 +720,7 @@ int main(int argc, char *argv[])
 	sigaction(SIGCHLD, &sa, NULL);
 	sigaction(SIGPIPE, &sa, NULL);
 
-	run_server(ctl, csk, isk, subclass, timeout);
+	run_server(ctl, csk, isk, subclass, nosdp, encrypt, timeout);
 
 	syslog(LOG_INFO, "Exit");
 
