@@ -24,7 +24,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: main.c,v 1.6 2004/06/29 08:53:29 holtmann Exp $
+ *  $Id: main.c,v 1.11 2004/12/25 17:43:18 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -47,9 +47,9 @@
 #include <sys/stat.h>
 
 #include <bluetooth/bluetooth.h>
-#include <bluetooth/l2cap.h>
 #include <bluetooth/hci.h>
 #include <bluetooth/hci_lib.h>
+#include <bluetooth/l2cap.h>
 #include <bluetooth/bnep.h>
 
 #include "pand.h"
@@ -62,6 +62,7 @@ static int  persist;
 static int  use_sdp = 1;
 static int  use_cache;
 static int  encrypt;
+static int  secure;
 static int  master;
 static int  cleanup;
 static int  search_duration = 10;
@@ -138,9 +139,10 @@ static int do_listen(void)
 		return -1;
 	}
 
+	memset(&l2a, 0, sizeof(l2a));
 	l2a.l2_family = AF_BLUETOOTH;
-	l2a.l2_psm    = htobs(BNEP_PSM);
-	l2a.l2_bdaddr = src_addr;
+	bacpy(&l2a.l2_bdaddr, &src_addr);
+	l2a.l2_psm = htobs(BNEP_PSM);
 
 	if (bind(sk, (struct sockaddr *) &l2a, sizeof(l2a))) {
 		syslog(LOG_ERR, "Bind failed. %s(%d)", strerror(errno), errno);
@@ -148,6 +150,7 @@ static int do_listen(void)
 	}
 
 	/* Setup L2CAP options according to BNEP spec */
+	memset(&l2o, 0, sizeof(l2o));
 	olen = sizeof(l2o);
 	if (getsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &l2o, &olen) < 0) {
 		syslog(LOG_ERR, "Failed to get L2CAP options. %s(%d)",
@@ -166,9 +169,10 @@ static int do_listen(void)
 	lm = 0;
 	if (master)
 		lm |= L2CAP_LM_MASTER;
-
 	if (encrypt)
 		lm |= L2CAP_LM_ENCRYPT;
+	if (secure)
+		lm |= L2CAP_LM_SECURE;
 
 	if (lm && setsockopt(sk, SOL_L2CAP, L2CAP_LM, &lm, sizeof(lm)) < 0) {
 		syslog(LOG_ERR, "Failed to set link mode. %s(%d)", strerror(errno), errno);
@@ -270,23 +274,24 @@ static int create_connection(char *dst, bdaddr_t *bdaddr)
 	}
 
 	/* Setup L2CAP options according to BNEP spec */
+	memset(&l2o, 0, sizeof(l2o));
 	olen = sizeof(l2o);
 	getsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &l2o, &olen);
 	l2o.imtu = l2o.omtu = BNEP_MTU;
 	setsockopt(sk, SOL_L2CAP, L2CAP_OPTIONS, &l2o, sizeof(l2o));
 
+	memset(&l2a, 0, sizeof(l2a));
 	l2a.l2_family = AF_BLUETOOTH;
-
-	/* Set local address */
-	l2a.l2_psm = 0;
-	l2a.l2_bdaddr = src_addr;
+	bacpy(&l2a.l2_bdaddr, &src_addr);
 
 	if (bind(sk, (struct sockaddr *) &l2a, sizeof(l2a)))
 		syslog(LOG_ERR, "Bind failed. %s(%d)", 
 				strerror(errno), errno);
 
+	memset(&l2a, 0, sizeof(l2a));
+	l2a.l2_family = AF_BLUETOOTH;
+	bacpy(&l2a.l2_bdaddr, bdaddr);
 	l2a.l2_psm = htobs(BNEP_PSM);
-	l2a.l2_bdaddr = *bdaddr;
 
 	if (!connect(sk, (struct sockaddr *) &l2a, sizeof(l2a)) && 
 			!bnep_create_connection(sk, role, service, netdev)) {
@@ -485,14 +490,15 @@ static struct option main_lopts[] = {
 	{ "killall",  0, 0, 'K' },
 	{ "role",     1, 0, 'r' },
 	{ "service",  1, 0, 'd' },
+	{ "ethernet", 1, 0, 'e' },
 	{ "device",   1, 0, 'i' },
-	{ "source",   1, 0, 'S' },
 	{ "nosdp",    0, 0, 'D' },
 	{ "list",     0, 0, 'l' },
 	{ "show",     0, 0, 'l' },
 	{ "nodetach", 0, 0, 'n' },
 	{ "persist",  2, 0, 'p' },
 	{ "encrypt",  0, 0, 'E' },
+	{ "secure",   0, 0, 'S' },
 	{ "master",   0, 0, 'M' },
 	{ "cache",    0, 0, 'C' },
 	{ "pidfile",  1, 0, 'P' },
@@ -500,10 +506,10 @@ static struct option main_lopts[] = {
 	{ 0, 0, 0, 0 }
 };
 
-static char main_sopts[] = "hsc:k:Kr:i:S:lnp::DQ::EMC::P:z";
+static char main_sopts[] = "hsc:k:Kr:e:i:lnp::DQ::ESMC::P:z";
 
 static char main_help[] = 
-	"PAN daemon version " VERSION " \n"
+	"Bluetooth PAN daemon version " VERSION " \n"
 	"Usage:\n"
 	"\tpand <options>\n"
 	"Options:\n"
@@ -516,10 +522,11 @@ static char main_help[] =
 	"\t--killall -K              Kill all PAN connections\n"
 	"\t--role -r <role>          Local PAN role (PANU, NAP, GN)\n"
 	"\t--service -d <role>       Remote PAN service (PANU, NAP, GN)\n"
-	"\t--device -i <name>        Network interface name\n"
-	"\t--source -S <bdaddr>      Source bdaddr\n"
+	"\t--ethernet -e <name>      Network interface name\n"
+	"\t--device -i <bdaddr>      Source bdaddr\n"
 	"\t--nosdp -D                Disable SDP\n"
 	"\t--encrypt -E              Enable encryption\n"
+	"\t--secure -S               Secure connection\n"
 	"\t--master -M               Become the master of a piconet\n"
 	"\t--nodetach -n             Do not become a daemon\n"
 	"\t--persist -p[interval]    Persist mode\n"
@@ -551,7 +558,6 @@ int main(int argc, char **argv)
 
 		case 'Q':
 			mode = CONNECT;
-			dst  = NULL;
 			if (optarg)
 				search_duration = atoi(optarg);
 			break;
@@ -565,10 +571,9 @@ int main(int argc, char **argv)
 		case 'K':
 			mode = KILL;
 			detach = 0;
-			dst  = NULL;
 			break;
 
-		case 'S':
+		case 'i':
 			src = strdup(optarg);
 			break;
 
@@ -588,11 +593,15 @@ int main(int argc, char **argv)
 			encrypt = 1;
 			break;
 
+		case 'S':
+			secure = 1;
+			break;
+
 		case 'M':
 			master = 1;
 			break;
 
-		case 'i':
+		case 'e':
 			strcpy(netdev, optarg);
 			break;
 
@@ -680,7 +689,7 @@ int main(int argc, char **argv)
 	}
 
 	openlog("pand", LOG_PID | LOG_NDELAY | LOG_PERROR, LOG_DAEMON);
-	syslog(LOG_INFO, "Bluetooth PAN daemon");
+	syslog(LOG_INFO, "Bluetooth PAN daemon version %s", VERSION);
 
 	if (src) {
 		src_dev = hci_devid(src);
@@ -700,6 +709,7 @@ int main(int argc, char **argv)
 		strncpy(cache.dst, dst, sizeof(cache.dst) - 1);
 		str2ba(dst, &cache.bdaddr);
 		cache.valid = 1;
+		free(dst);
 	}
 
 	switch (mode) {
