@@ -23,7 +23,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: hstest.c,v 1.4 2004/04/28 12:44:33 holtmann Exp $
+ *  $Id: hstest.c,v 1.6 2004/12/25 17:43:20 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -98,6 +98,7 @@ static int sco_connect(bdaddr_t *src, bdaddr_t *dst, uint16_t *handle, uint16_t
 	memset(&addr, 0, sizeof(addr));
 	addr.sco_family = AF_BLUETOOTH;
 	bacpy(&addr.sco_bdaddr, src);
+
 	if (bind(s, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
 		close(s);
 		return -1;
@@ -106,18 +107,23 @@ static int sco_connect(bdaddr_t *src, bdaddr_t *dst, uint16_t *handle, uint16_t
 	memset(&addr, 0, sizeof(addr));
 	addr.sco_family = AF_BLUETOOTH;
 	bacpy(&addr.sco_bdaddr, dst);
+
 	if (connect(s, (struct sockaddr *)&addr, sizeof(addr)) < 0 ){
 		close(s);
 		return -1;
 	}
 
+	memset(&conn, 0, sizeof(conn));
 	size = sizeof(conn);
+
 	if (getsockopt(s, SOL_SCO, SCO_CONNINFO, &conn, &size) < 0) {
 		close(s);
 		return -1;
 	}
 
+	memset(&opts, 0, sizeof(opts));
 	size = sizeof(opts);
+
 	if (getsockopt(s, SOL_SCO, SCO_OPTIONS, &opts, &size) < 0) {
 		close(s);
 		return -1;
@@ -148,7 +154,7 @@ int main(int argc, char *argv[])
 
 	fd_set rfds;
 	struct timeval timeout;
-	unsigned char buf[2048];
+	unsigned char buf[2048], *p;
 	int maxfd, sel, rlen, wlen;
 
 	bdaddr_t local;
@@ -274,7 +280,15 @@ int main(int argc, char *argv[])
 					switch (mode) {
 					case PLAY:
 						rlen = read(fd, buf, rlen);
-						wlen = write(sd, buf, rlen);
+
+						wlen = 0; 
+						p = buf;
+						while (rlen > sco_mtu) {
+						        wlen += write(sd, p, sco_mtu);
+						        rlen -= sco_mtu;
+						        p += sco_mtu;
+						}
+						wlen += write(sd, p, rlen);
 						break;
 					case RECORD:
 						wlen = write(fd, buf, rlen);
