@@ -25,7 +25,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: hciconfig.c,v 1.51 2005/01/11 13:33:14 holtmann Exp $
+ *  $Id: hciconfig.c,v 1.53 2005/01/31 05:45:39 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -248,6 +248,10 @@ static void cmd_up(int ctl, int hdev, char *opt)
 						hdev, strerror(errno), errno);
 		exit(1);
 	}
+	if (ioctl(ctl, HCIGETDEVINFO, (void *) &di) < 0)
+		return;
+	if (hci_test_bit(HCI_RAW, &di.flags))
+		return;
 	cmd_scan(ctl, hdev, "piscan");
 }
 
@@ -1095,7 +1099,8 @@ static void print_dev_info(int ctl, struct hci_dev_info *di)
 	printf("\tTX bytes:%d acl:%d sco:%d commands:%d errors:%d\n",
 		st->byte_tx, st->acl_tx, st->sco_tx, st->cmd_tx, st->err_tx);
 
-	if (all) {
+	if (all && !hci_test_bit(HCI_RAW, &di->flags) &&
+			bacmp(&di->bdaddr, BDADDR_ANY)) {
 		print_dev_features(di, 0);
 		print_pkt_type(di);
 		print_link_policy(di);
