@@ -23,7 +23,7 @@
  *  SOFTWARE IS DISCLAIMED.
  *
  *
- *  $Id: hid2hci.c,v 1.4 2004/08/12 13:17:22 holtmann Exp $
+ *  $Id: hid2hci.c,v 1.5 2004/11/29 15:02:25 holtmann Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -219,6 +219,8 @@ static int switch_logitech(struct device_info *devinfo)
 static struct device_id device_list[] = {
 	{ HCI, 0x0a12, 0x1000, switch_hidproxy },
 	{ HID, 0x0a12, 0x0001, switch_hidproxy },
+	{ HCI, 0x0458, 0x1000, switch_hidproxy },
+	{ HID, 0x0458, 0x003f, switch_hidproxy },
 	{ HCI, 0x05ac, 0x1000, switch_hidproxy },
 	{ HID, 0x05ac, 0x8203, switch_hidproxy },
 	{ HCI, 0x046d, 0xc703, switch_logitech },
